<?php
/**
 * Theme setup class.
 *
 * Set up theme defaults and registers support for various WordPress features.
 *
 * Note that this function is hooked into the after_setup_theme hook, which
 * runs before the init hook. The init hook is too late for some features, such
 * as indicating support post thumbnails.
 *
 */

class Wild_Book_Setup {

  // Theme version
  protected $version;
  
  // Theme unique id
  protected $theme_slug;
  
  public function __construct() {
    $this->version    = '1.6.4';
    $this->theme_slug = 'wildbook';
  }
  
  /**
   * Setup theme. Add actions, filters, features etc.
   *
   * @since Wild Book 1.0
   */   
  public function setup() {
    global $content_width;

    /**
     * Set up the content width value based on the theme's design.
     */
    if ( ! isset( $content_width ) ) {
      $content_width = 980;
    }
  
    /*
     * Make theme available for translation.
     *
     * Translations can be added to the /languages/ directory.
     */
    load_theme_textdomain( 'wildbook', get_template_directory() . '/languages' );
  
    // Add RSS feed links to <head> for posts and comments.
    add_theme_support( 'automatic-feed-links' );

    // Enable support for Post Thumbnails, and declare two sizes.
    add_theme_support( 'post-thumbnails' );
    set_post_thumbnail_size( 940, 590, true );
    add_image_size( 'post-thumbnail-not-crop', 940, 9999, false );
    add_image_size( 'medium-thumbnail', 500, 330, true );
    add_image_size( 'medium-thumbnail-not-crop', 500, 9999, false );
    add_image_size( 'wdb-medium-square', 500, 500, true );
    
    // Register navigation menus.
    register_nav_menus( array(
      'primary'   => esc_html__( 'Sidebar menu', 'wildbook' ),
      'secondary' => esc_html__( 'Top primary menu', 'wildbook' )
    ) );
    
    /*
     * Switch default core markup for search form, comment form, and comments
     * to output valid HTML5.
     */
    add_theme_support( 'html5', array(
      'search-form', 'comment-form', 'comment-list', 'gallery', 'caption'
    ) );
    
    /*
     * Enable support for Post Formats.
     *
     * See: https://codex.wordpress.org/Post_Formats
     */
    add_theme_support( 'post-formats', array( 'video', 'audio' ) );
    
    // Add document title tag to HTML <head>.
    add_theme_support( 'title-tag' );
    
    // Setup the WordPress core custom header image.  
    add_theme_support( 'custom-header', array( 
      'width'       => 230,
      'height'      => 230,
      'flex-height' => true,
      'flex-width'  => true
    ) );
    
    // Setup the WordPress core custom background feature.    
    add_theme_support( 'custom-background', array( 
      'default-color'      => '5f5f5f',
      'default-image'      => get_template_directory_uri() . '/images/background.jpg',
      'default-preset'     => 'default',
      'default-position-x' => 'center',
      'default-position-y' => 'center',
      'default-size'       => 'cover',
      'default-repeat'     => 'no-repeat',
      'default-attachment' => 'scroll',
      'wp-head-callback'   => array( $this, 'custom_background_cb' )
    ) );
    
    // Add public scripts and styles
    add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
    add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_styles' ), 10 );
    
    // Widgets init
    add_action( 'widgets_init', array( $this, 'widgets_init' ) );
    
    // Modify Read More link
    add_filter( 'the_content_more_link', array( $this, 'modify_read_more_link' ) );
    
    // Modify search form
    add_filter( 'get_search_form', array( $this, 'search_form' ) );
    
    // Set new excerpt length
    add_filter( 'excerpt_length', array( $this, 'custom_excerpt_length' ), 999 );
    
    // Set new excerpt more text
    add_filter('excerpt_more', array( $this, 'custom_excerpt_more' ));
    
    // This theme uses its own gallery styles.
    add_filter( 'use_default_gallery_style', '__return_false' );
    
    // Add classes to body tag.
    add_filter( 'body_class', array( $this, 'add_body_class' ) );
    
    add_action( 'customize_save_after', array( $this, 'reset_inline_style_cache' ) );
    
    /*
     * This theme styles the visual editor to resemble the theme style,
     * specifically font, colors, icons, and column width.
     */
    add_editor_style( array( 'css/editor-style.css', $this->fonts_url() ) );
    
    // Adding to TinyMCE editor drop down menu with custom styles.
    add_filter( 'mce_buttons_2',         array( $this, 'wpb_mce_buttons_2' ) );
    add_filter( 'tiny_mce_before_init', array( $this, 'insert_formats_to_editor' ) );
    
    // Protection against duplication "portfolio" slug
    add_filter( 'wp_unique_post_slug_is_bad_hierarchical_slug', array( $this, 'portfolio_is_bad_hierarchical_slug' ), 10, 4 );
    add_filter( 'wp_unique_post_slug_is_bad_flat_slug', array( $this, 'portfolio_is_bad_flat_slug' ), 10, 3 );
    
    // Modification portfolio archive title
    add_filter( 'get_the_archive_title', array( $this, 'archive_title' ), 10, 1 );

    // Comment form fields
    add_filter( 'comment_form_default_fields', array( $this, 'comment_form_default_fields' ), 10, 1 );
  }
  
  /**
   * Register and enqueue theme scripts.
   *
   * @since Wild Book 1.0
   */  
  public function enqueue_scripts() {
    if ( is_singular() && comments_open() && get_option( 'thread_comments' ) ) {
      wp_enqueue_script( 'comment-reply' );
    }

    wp_register_script( 'owl-carousel', get_template_directory_uri() . '/js/owl.carousel.min.js', array( 'jquery' ), '2.2.1', true );
    
    if( get_theme_mod( 'winter_mode', 0 ) ) {
      wp_enqueue_script( 'jquery-snowfall', get_template_directory_uri() . '/js/snowfall.jquery.js', array( 'jquery' ), '3.1.8', true );
    }
    
    wp_enqueue_script( 'wait-for-images', get_template_directory_uri() . '/js/jquery.waitforimages.min.js', array( 'jquery' ), '2.1.0', true );

    wp_enqueue_script( 'fitvids',              get_template_directory_uri() . '/js/jquery.fitvids.min.js', array( 'jquery' ), '1.1', true );
    wp_enqueue_script( 'wild-book-functions',  get_template_directory_uri() . '/js/functions.min.js', array( 'jquery' ), $this->version, true );     
  }
  
  /**
   * Register and enqueue theme styles.
   *
   * @since Wild Book 1.0
   */    
  public function enqueue_styles() {
    // Add fonts, used in the main stylesheet.
    wp_enqueue_style( 'wild-book-fonts', $this->fonts_url(), array(), $this->version );

    wp_register_style( 'owl-carousel', get_template_directory_uri() . '/css/owl.carousel.min.css', array(), '2.2.1' );

    // Font Awesome icons font style
    wp_enqueue_style( 'font-awesome', get_template_directory_uri() . '/css/font-awesome.min.css', array(), '4.7' );  

    // Load our main stylesheet.
    wp_enqueue_style( 'wild-book-style', get_stylesheet_uri(), array(), $this->version );

    if( get_bloginfo( 'language' ) == 'ja' ) {
       wp_enqueue_style( 'wild-book-style-japanese', get_template_directory_uri() . '/css/style-japanese.css', array(), $this->version );      
    }

    // Add customizer css.
    wp_add_inline_style( 'wild-book-style', $this->inline_style() );
  }
  
  /**
   * Register Google fonts for Wild Book.
   *
   * @since Wild Book 1.0
   *
   * @return string Google fonts URL for the theme.
   */  
  public function fonts_url() {
    $fonts_url = '';
    $fonts     = array();
    $subsets   = 'latin,latin-ext';

    /* translators: If there are characters in your language that are not supported by Lora, translate this to 'off'. Do not translate into your own language. */
    if ( 'off' !== _x( 'on', 'Lora font: on or off', 'wildbook' ) ) {
      $fonts[] = 'Lora:400,700,400italic,700italic';
    }
    
    /* translators: If there are characters in your language that are not supported by Raleway, translate this to 'off'. Do not translate into your own language. */
    if ( 'off' !== _x( 'on', 'Raleway font: on or off', 'wildbook' ) ) {
      $fonts[] = 'Raleway:500,600';
    }
    
    /* Use Open Sans font if language that are not supported by Raleway */
    if ( 'on' == _x( 'off', 'Open Sans font: on or off', 'wildbook' ) ) {
      $fonts[] = 'Open Sans';
    }
    
    /* translators: To add an additional character subset specific to your language, translate this to 'greek', 'cyrillic', 'devanagari' or 'vietnamese'. Do not translate into your own language. */
    $subset = _x( 'no-subset', 'Add new subset (greek, cyrillic, devanagari, vietnamese)', 'wildbook' );

    if ( 'cyrillic' == $subset ) {
      $subsets .= ',cyrillic,cyrillic-ext';
    } elseif ( 'greek' == $subset ) {
      $subsets .= ',greek,greek-ext';
    } elseif ( 'devanagari' == $subset ) {
      $subsets .= ',devanagari';
    } elseif ( 'vietnamese' == $subset ) {
      $subsets .= ',vietnamese';
    }

    if ( $fonts ) {
      $fonts_url = esc_url( add_query_arg( array(
        'family' => urlencode( implode( '|', $fonts ) ),
        'subset' => urlencode( $subsets ),
      ), '//fonts.googleapis.com/css' ) );
    }

    return $fonts_url;
  }
  
  /**
   * Inline style. Used in wp_add_inline_style.
   *
   * @since Wild Book 1.0
   */  
  public function inline_style() {
    if( is_customize_preview() )
      return $this->get_inline_style();
      
    $inline_style = get_transient( 'wdb_inline_style' );
    
    if( $inline_style === false ) {
      $inline_style = $this->get_inline_style();
      set_transient( 'wdb_inline_style', $inline_style );
    }

    $inline_style .= '
        .site {
          visibility:hidden;
        }
        .loaded .site {
          visibility:visible;
        }
      ';

    if ( is_singular() && comments_open() ) {
      $inline_style .= '
        .comment-list .bypostauthor .fn:after {
          content: "' . esc_html__( 'Author', 'wildbook' ) . '";
        }';
    }
    
    return $inline_style;
  }
  
  /**
   * Get inline style css.
   *
   * @since Wild Book 1.0
   *
   * @return string Css with custom styles.
   */  
  public function get_inline_style() {
    $css = '';

    $logo_size = absint( get_theme_mod( 'logo_size' ) );

    if( $logo_size ){
      $css .= "
        .header-image img {
          max-width: {$logo_size}em;
          height: auto;
        }";
    }

    $logo_margin_bottom = absint( get_theme_mod( 'logo_margin_bottom' ) );
    
    if( $logo_margin_bottom ){
      $css .= "
        .header-image {
          margin-bottom: {$logo_margin_bottom}em;
        }";
    }

    // Sidebar colors
    
    $sidebar_text_color = get_theme_mod( 'side_text_color' );
    
    if( $sidebar_text_color ) {
      $css .= '
        .sidebar {
          color: ' . $sidebar_text_color . ';
        }';
    }

    $sidebar_link_color = get_theme_mod( 'main_nav_link_color' );
    
    if( $sidebar_link_color ) {
      $css .= "
        .sidebar a {
          color: {$sidebar_link_color};
        }";
    }
    
    $sidebar_link_color_hover = get_theme_mod( 'main_nav_link_hover_color' );
    
    if( $sidebar_link_color_hover ) {
      $css .= "
        .sidebar a:hover {
          color: {$sidebar_link_color_hover};
        }";
    }

    $sidebar_icons_color = get_theme_mod( 'side_icons_color' );
    
    if( $sidebar_icons_color ) {
      $css .= "
        .toggle-buttons .search-toggle {
          color: {$sidebar_icons_color};
        }
        .nav-toggle .menu-icon, 
        .nav-toggle .menu-icon:before, 
        .nav-toggle .menu-icon:after {
          background-color: {$sidebar_icons_color};
        }";
    }

    $sidebar_icons_hover_color = get_theme_mod( 'side_icons_hover_color' );
    
    if( $sidebar_icons_hover_color ) {
      $css .= "
        .toggle-buttons .search-toggle:hover {
          color: {$sidebar_icons_hover_color};
        }
        .nav-toggle:hover .menu-icon, 
        .nav-toggle:hover .menu-icon:before, 
        .nav-toggle:hover .menu-icon:after {
          background-color: {$sidebar_icons_hover_color};
        }";
    }

    $sidebar_social_icons_color = get_theme_mod( 'side_social_icons_color' );
    
    if( $sidebar_social_icons_color ) {
      $css .= '
        .sidebar .social-profiles {
          color: ' . $sidebar_social_icons_color . ';
        }
        .sidebar .social-profiles a {
          border-color: ' . $this->hex2rgba( $sidebar_social_icons_color, 0.3, true ) . ';
        }
        .sidebar .social-profiles a:hover {
          border-color: ' . $this->hex2rgba( $sidebar_social_icons_color, 0.5, true ) . ';
        }';
    }

    // Navigation menu colors

    $top_nav_background_color = get_theme_mod( 'right_nav_background_color' );
    
    if( $top_nav_background_color ) {
      $css .= "
        .top-nav {
          background-color: {$top_nav_background_color};
        }";
    }

    $top_nav_links_color = get_theme_mod( 'right_nav_links_color' );
    
    if( $top_nav_links_color ) {
      $css .= "
        @media (min-width: 992px) {
          .top-nav .nav-menu > li > a {
            color: {$top_nav_links_color};
          }
        }
        .top-nav-toggle {
          color: {$top_nav_links_color};
        }";
    }

    $top_nav_links_hover_color = get_theme_mod( 'right_nav_links_hover_color' );
    
    if( $top_nav_links_hover_color ) {
      $css .= "
        @media (min-width: 992px) {
          .top-nav .nav-menu > li > a:hover {
            color: {$top_nav_links_hover_color};
          }
        }
        .top-nav-toggle:hover {
          color: {$top_nav_links_hover_color};
        }";
    }

    $top_nav_dropdown_background_color = get_theme_mod( 'right_nav_dropdown_background_color' );
    
    if( $top_nav_dropdown_background_color ) {
      $css .= "
        @media (min-width: 992px) { 
          .top-nav .nav-menu .sub-menu {
            background-color: {$top_nav_dropdown_background_color};
          }
        }";
    }

    $top_nav_dropdown_links_color = get_theme_mod( 'right_nav_dropdown_links_color' );
    
    if( $top_nav_dropdown_links_color ) {
      $css .= "
        @media (min-width: 992px) { 
          .top-nav .nav-menu .sub-menu > li > a {
            color: {$top_nav_dropdown_links_color};
          }
        }";
    }

    $top_nav_dropdown_links_hover_color = get_theme_mod( 'right_nav_dropdown_links_hover_color' );
    
    if( $top_nav_dropdown_links_hover_color ) {
      $css .= "
        @media (min-width: 992px) { 
          .top-nav .nav-menu .sub-menu > li > a:hover {
            color: {$top_nav_dropdown_links_hover_color};
          }
        }";
    }

    $top_nav_dividers_color = get_theme_mod( 'right_nav_dot_color' );
    
    if( $top_nav_dividers_color ) {
      $css .= "
        .top-nav .nav-menu > li > a:after {
          background-color: {$top_nav_dividers_color};
        }";
    }

    $top_nav_border_color = get_theme_mod( 'right_nav_border_color' );
    
    if( $top_nav_border_color ) {
      $css .= "
        @media (min-width: 992px) { 
          .top-nav {
            border-color: {$top_nav_border_color};
          }
        }
        @media (max-width: 991px) { 
          .top-nav-toggle {
            border-color: {$top_nav_border_color};
          }
        }
        ";
    }

    // Content area colors

    $content_area_background_color = get_theme_mod( 'content_area_background_color' );

    if( $content_area_background_color ) {
      $css .= "
        .site-content,
        blockquote:before, 
        blockquote:after {
          background-color: {$content_area_background_color};
        }";
    }

    $content_area_text_color = get_theme_mod( 'content_area_text_color' );

    if( $content_area_text_color ) {
      $css .= '
        body,
        a {
          color: ' . $content_area_text_color . ';
        }
        a:hover {
          color: ' . $this->hex2rgba( $content_area_text_color, 0.75, true ) . '
        }
        hr,
        pre,
        .text-box {
          border-color: ' . $this->hex2rgba( $content_area_text_color, 0.15, true ) . ';
        }
        table, th, td {
          border-color: ' . $this->hex2rgba( $content_area_text_color, 0.15, true ) . '
        }
        blockquote:before, 
        blockquote:after {
          color: ' . $this->hex2rgba( $content_area_text_color, 0.15, true ) . ';
        }
        table, th, td {
          border-color: ' . $this->hex2rgba( $content_area_text_color, 0.15, true ) . '
        }
        .cat-links:after {
          background-color: ' . $this->hex2rgba( $content_area_text_color, 0.15, true ) . ';
        }
        .comment-reply-link,
        .comment-reply-link:hover {
          color: ' . $content_area_text_color . ';
        }
        .comment-reply-link,
        .wdb-widget-about-me .social-profiles a {
          border-color: ' . $this->hex2rgba( $content_area_text_color, 0.17, true ) . ';
        }
        .comment-reply-link:hover,
        .wdb-widget-about-me .social-profiles a:hover {
          border-color: ' . $this->hex2rgba( $content_area_text_color, 0.35, true ) . ';
        }';
    }

    $content_area_secondary_text_color = get_theme_mod( 'content_area_secondary_text_color' );

    if( $content_area_secondary_text_color ) {
      $css .= '
        .post-meta, 
        .post-date, 
        .comment-meta, 
        .entry-meta .portfolio-cats a,
        .comment-list .bypostauthor .fn:after,
        .post-navigation .nav-meta,
        .widget_archive ul, 
        .widget_categories ul, 
        .widget_recent_comments ul,
        caption,
        .wp-caption-text, 
        .entry-caption,
        .tags-list a, 
        .tagcloud a,
        .tags-list a:hover, 
        .tagcloud a:hover {
          color: ' . $content_area_secondary_text_color . ';
        }
        .post-meta + .post-meta:before {
          color: ' . $this->hex2rgba( $content_area_secondary_text_color, 0.75, true ) . ';
        }
        .tags-list a, 
        .tagcloud a {
          border-color: ' . $this->hex2rgba( $content_area_secondary_text_color, 0.2, true ) . ';
        }
        .tags-list a:hover, 
        .tagcloud a:hover {
          border-color: ' . $this->hex2rgba( $content_area_secondary_text_color, 0.5, true ) . ';
        }';
    }

    $content_area_headings_color = get_theme_mod( 'content_area_headings_color' );

    if( $content_area_headings_color ) {
      $css .= '
        h1,
        h2,
        h3,
        h4,
        h5,
        h6 {
          color: ' . $content_area_headings_color . ';
        }
        .entry-title a:hover {
          color: ' . $this->hex2rgba( $content_area_headings_color, 0.75, true ) . '
        }';
    }

    $link_color = get_theme_mod( 'link_color' );
    
    if( $link_color ) {
      $css .= "
        .taxonomy-description a,
        .entry-meta a,
        .page-content a,
        .entry-content a,
        .author-info a,
        .comment-navigation a,
        .image-navigation a,
        .comment-content a,
        .pingback .comment-body > a,
        .comment-meta a,
        .logged-in-as a,
        .widget_calendar a,
        .textwidget a {
          color: {$link_color};
        }";
    }
    
    $link_color_hover = get_theme_mod( 'link_color_hover' );
    
    if( $link_color_hover ) {
      $css .= "
        .taxonomy-description a:hover,
        .cat-links a:hover,
        .entry-meta a:hover,
        .page-content a:hover,
        .entry-content a:hover,
        .author-info a:hover,
        .comment-navigation a:hover,
        .image-navigation a:hover,
        .comment-content a:hover,
        .comment-meta a:hover,
        .logged-in-as a:hover,
        .widget_calendar a:hover,
        .textwidget a:hover,
        .portfolio-grid .portfolio-cats a:hover {
          color: {$link_color_hover};
        }";
    }
    
    $button_background_color = get_theme_mod( 'button_background_color' );
    
    if( $button_background_color ) {
      $css .= "
        button,
        input[type='button'],
        input[type='reset'],
        input[type='submit'],
        .page-numbers:hover,
        .page-numbers.current {
          background: {$button_background_color};
        }";
    }
    
    $button_background_color_hover = get_theme_mod( 'button_background_color_on_hover' );
    
    if( $button_background_color_hover ) {
      $css .= "
        button:hover,
        input[type='button']:hover,
        input[type='reset']:hover,
        input[type='submit']:hover {
          background: {$button_background_color_hover};
        }";
    }
    
    $button_text_color = get_theme_mod( 'button_text_color' );
    
    if( $button_text_color ) {
      $css .= "
        button,
        input[type='button'],
        input[type='reset'],
        input[type='submit'],
        .page-numbers:hover,
        .page-numbers.current {
          color: {$button_text_color};
        }";
    }
    
    $share_button_background = get_theme_mod( 'share_button_background' );
    
    if( $share_button_background ) {
      $css .= "
        .share-buttons a {
          background: {$share_button_background};
        }";
    }
    
    $share_button_background_on_hover = get_theme_mod( 'share_button_background_on_hover' );
    
    if( $share_button_background_on_hover ) {
      $css .= "
        .share-buttons a:hover {
          background: {$share_button_background_on_hover};
        }";
    }
    
    $share_button_icon_color = get_theme_mod( 'share_button_icon_color' );
    
    if( $share_button_icon_color ) {
      $css .= "
        .share-buttons a {
          color: {$share_button_icon_color};
        }";
    }
    
    $share_button_icon_color_on_hover = get_theme_mod( 'share_button_icon_color_on_hover' );
    
    if( $share_button_icon_color_on_hover ) {
      $css .= "
        .share-buttons a:hover {
          color: {$share_button_icon_color_on_hover};
        }";
    }

    $dividers_color = get_theme_mod( 'dividers_color' );
    
    if( $dividers_color ) {
      $css .= "
        .navigation,
        .inner-box,
        .content-area  {
          border-color: {$dividers_color};
        }";
    }
    
    $preloader_color = get_theme_mod( 'preloader_color' );
    
    if( $preloader_color ) {
      $css .= "
        .double-bounce1, 
        .double-bounce2 {
          background-color: {$preloader_color};
        }";
    }

    $sidebar_width = absint( get_theme_mod( 'sidebar_width' ) );

    if ( $sidebar_width ) {
      $css .= sprintf( '
        @media (min-width: 992px) { 
          .sidebar {
            width: %1$d%%;
          }
          .site-content {
            margin-left: %1$d%%;
          }
          .sticky-nav .top-nav {
            left: %1$d%%;
          }
          .rtl .site-content {
            margin-right: %1$d%%;
            margin-left: 0;
          }
          .rtl.sticky-nav .top-nav {
            right: %1$d%%;
            left: 0;
          }
          .sidebar-right .site-content {
            margin-right: %1$d%%;
          }
          .sidebar-right.sticky-nav .top-nav {
            right: %1$d%%;
          }
          .rtl.sidebar-right .site-content {
            margin-left: %1$d%%;
          }
          .rtl.sidebar-right.sticky-nav .top-nav {
            left: %1$d%%;
          }
        }',
        $sidebar_width
      );
    }
    
    $custom_css = get_theme_mod( 'wdb_custom_css' );
    
    if( $custom_css ) {
       $css .= wp_filter_nohtml_kses( $custom_css );
    }
    
    return $css;
  }

  /**
   * Convert hex to rgba.
   *
   * @since Wild Book 1.4.8
   *
   * @return string/array Converted color.
   */  
  public function hex2rgba( $color, $opacity = 1, $css = false ) {
    if( empty( $color ) )
      return;

    $color = str_replace( '#', '', $color );

    if ( strlen( $color ) == 6 ) {
      $r = hexdec( $color[0] . $color[1] );
      $g = hexdec( $color[2] . $color[3] );
      $b = hexdec( $color[4] . $color[5] );
    } elseif ( strlen( $color) == 3 ) {
      $r = hexdec( $color[0] . $color[0] );
      $g = hexdec( $color[1] . $color[1] );
      $b = hexdec( $color[2] . $color[2] );
    } else {
      return false;
    }

    $opacity = floatval( $opacity );

    if( $css )
      return 'rgba( ' . $r . ', ' . $g . ', ' . $b . ', ' . $opacity . ' )';
    else 
      return compact( $r, $g, $b, $opacity );
  }
  
  /**
   * Reset custom styles cache.
   *
   * @since Wild Book 1.4.3
   */
  function reset_inline_style_cache() {
    delete_transient( 'wdb_inline_style' );
  }

  /**
   * Register widget area.
   *
   * @since Wild Book 1.0
   *
   * @link https://codex.wordpress.org/Function_Reference/register_sidebar
   */
  public function widgets_init() {
    require get_template_directory() . '/inc/widgets.php';
    register_widget( 'Wild_Book_About_Me' );
    register_widget( 'Wild_Book_Instagram_Widget' );
    register_widget( 'Wild_Book_Recent_Posts' );
  
    // Register sidebar
    register_sidebar( array(
      'name'          => esc_html__( 'First Footer Widget Area', 'wildbook' ),
      'id'            => 'widget-area-1',
      'description'   => esc_html__( 'Add widgets here to appear in your footer.', 'wildbook' ),
      'before_widget' => '<aside id="%1$s" class="widget %2$s">',
      'after_widget'  => '</aside>',
      'before_title'  => '<h3 class="widget-title">',
      'after_title'   => '</h3>',
    ) );
    
    register_sidebar( array(
      'name'          => esc_html__( 'Second Footer Widget Area', 'wildbook' ),
      'id'            => 'widget-area-2',
      'description'   => esc_html__( 'Add widgets here to appear in your footer.', 'wildbook' ),
      'before_widget' => '<aside id="%1$s" class="widget %2$s">',
      'after_widget'  => '</aside>',
      'before_title'  => '<h3 class="widget-title">',
      'after_title'   => '</h3>',
    ) );
  }
  
  /**
   * More link modification.
   *
   * @since Wild Book 1.0
   *
   * @return string Modified more link.
   */  
  public function modify_read_more_link() {
    $more_link_text = get_theme_mod( 'more_link_title' ) ? get_theme_mod( 'more_link_title' ) : esc_html__( 'Continue reading &rarr;', 'wildbook' );
    
    if( get_theme_mod( 'display_share_buttons', 1 ) ) {
      $more_link = sprintf( '<footer class="entry-more"><div class="more-wrap"><a class="more-link" href="%s">%s</a></div>%s</footer>', esc_url( get_permalink() ), esc_html( $more_link_text ), wdb_get_share_buttons() );
    } else {
      $more_link = sprintf( '<footer class="entry-more"><a class="more-link" href="%s">%s</a></footer>', esc_url( get_permalink() ), esc_html( $more_link_text ) );
    }
      
    return $more_link;
  }
  
  public function custom_background_cb() {
    // $background is the saved custom image, or the default image.
    $background_image_url = set_url_scheme( get_background_image() );
    $mobile_background_image_url = get_theme_mod( 'sidebar_background_image_for_mobile' );

    $custom_page_background = false;
    
    if( is_singular() ) {
      global $post;
      
      $background_image_id = get_post_meta( $post->ID, 'left_side_background_image_id', true );
      
      if( $background_image_id ) {
        $post_background_image = wp_get_attachment_image_src( $background_image_id, 'full' );
        
        if( $post_background_image ) {
          $custom_page_background = true;
          $background_image_url = $post_background_image[0];
        }
      }
    }

    if( is_home() && get_option( 'page_for_posts' ) ) {
      $background_image_id = get_post_meta( get_option( 'page_for_posts' ), 'left_side_background_image_id', true );
      
      if( $background_image_id ) {
        $post_background_image = wp_get_attachment_image_src( $background_image_id, 'full' );
        
        if( $post_background_image ) {
          $custom_page_background = true;
          $background_image_url = $post_background_image[0];
        }
      }
    }

    if( is_archive() && ! is_post_type_archive() && ! is_date() ) {
      $term = get_queried_object();
      
      if( isset( $term->term_id ) ) {
        $term_id = $term->term_id;
        
        $background_image_taxonomies = get_option( 'wdb_background_image_taxonomies' );
        $background_image_id = isset( $background_image_taxonomies[ $term_id ] ) ? $background_image_taxonomies[ $term_id ] : '';
        
        if( $background_image_id ) {
          $taxonomy_background_image = wp_get_attachment_image_src( $background_image_id, 'full' );
          
          if( $taxonomy_background_image ) {
            $custom_page_background = true;
            $background_image_url = $taxonomy_background_image[0];
          }
        }
      }
    }

    // $color is the saved custom color.
    // A default has to be specified in style.css. It will not be printed here.
    $color = get_background_color();

    if ( $color === get_theme_support( 'custom-background', 'default-color' ) ) {
      $color = false;
    }

    if ( ! $background_image_url && ! $mobile_background_image_url && ! $color ) {
      if ( is_customize_preview() ) {
        echo '<style type="text/css" id="custom-background-css"></style>';
      }
      return;
    }

    $style = '';

    if( $color ) {
      $style .= sprintf( '
        .sidebar {
          background-color: #%s;
        }',
        $color
      );
    }

    if ( $background_image_url || $mobile_background_image_url ) {
      // Background Position.
      $position_x = get_theme_mod( 'background_position_x', get_theme_support( 'custom-background', 'default-position-x' ) );
      $position_y = get_theme_mod( 'background_position_y', get_theme_support( 'custom-background', 'default-position-y' ) );

      if ( ! in_array( $position_x, array( 'left', 'center', 'right' ), true ) ) {
        $position_x = 'left';
      }

      if ( ! in_array( $position_y, array( 'top', 'center', 'bottom' ), true ) ) {
        $position_y = 'top';
      }

      // Background Size.
      $size = get_theme_mod( 'background_size', get_theme_support( 'custom-background', 'default-size' ) );

      if ( ! in_array( $size, array( 'auto', 'contain', 'cover' ), true ) ) {
        $size = 'auto';
      }

      // Background Repeat.
      $repeat = get_theme_mod( 'background_repeat', get_theme_support( 'custom-background', 'default-repeat' ) );

      if ( ! in_array( $repeat, array( 'repeat-x', 'repeat-y', 'repeat', 'no-repeat' ), true ) ) {
        $repeat = 'repeat';
      }

      // Background Scroll.
      $attachment = get_theme_mod( 'background_attachment', get_theme_support( 'custom-background', 'default-attachment' ) );

      if ( 'fixed' !== $attachment ) {
        $attachment = 'scroll';
      }

      if( $background_image_url ) {
        $style .= sprintf( '
          .sidebar {
            background-image: url( %s ) %s;
          }
          .sidebar:before {
            display: block;
          }',
          esc_url( $background_image_url ),
          $custom_page_background ? '!important' : ''
        );
      }

      if( $mobile_background_image_url ) {
        $style .= sprintf( '
          @media (max-width: 991px) {
            .sidebar {
              background-image: url( %s );
            }
            .sidebar:before {
              display: block;
            }
          }',
          esc_url( $mobile_background_image_url )
        );     
      }

      $style .= sprintf( '
          .sidebar {
            background-position: %s %s;
            background-size: %s;
            background-repeat: %s;
            background-attachment: %s;
          }',
        $position_x,
        $position_y,
        $size,
        $repeat,
        $attachment
      );

      $blackout_opacity = get_theme_mod( 'background_dark_mask_opacity', 0.5 );

      if( is_numeric( $blackout_opacity ) && $blackout_opacity >= 0 && $blackout_opacity <= 1 ) {
        $style .= sprintf( '
          .sidebar:before {
            opacity: %s;
          }',
          $blackout_opacity
        );
      }
    }

    if( $style ) {
      printf( '<style type="text/css" id="custom-background-css">%s</style>', $style );     
    }
  }
  
  /**
   * Search form modification.
   *
   * @since Wild Book 1.0
   *
   * @return string HTML with search form.
   */   
  public function search_form( $form ) {
    $form = '
      <form role="search" method="get" class="search-form" action="' . esc_url( home_url( '/' ) ) . '">
        <div class="search-wrap">
          <label>
            <input type="search" class="search-field" placeholder="' . esc_attr_x( 'Type keywords...', 'placeholder', 'wildbook' ) . '" value="' . get_search_query() . '" name="s" title="' . esc_attr_x( 'Search for:', '', 'wildbook' ) . '" />
          </label>
          <button type="submit" class="search-submit"><i class="fa fa-search"></i></button>
        </div>
			</form>';
    return $form;
  }
  
  /**
   * Modification default excerpt lenght.
   *
   * @since Wild Book 1.0
   *
   * @return int Excerpt lenght.
   */   
  public function custom_excerpt_length( $length ) {
    return 20;
  }
  
  /**
   * Modification default excerpt more.
   *
   * @since Wild Book 1.0
   *
   * @return string Excerpt more text.
   */   
  public function custom_excerpt_more( $more ) {
    return '...';
  }
  
  /**
   * Add classes to body tag.
   *
   * @since Wild Book 1.1
   *
   * @return array Body classes array.
   */    
  public function add_body_class( $classes ) {
    if( ! get_theme_mod( 'show_preloader_screen', 1 ) ) {
      $classes[] = 'loaded preloader-off';
    }

    if( get_search_query() )
      $classes[] = 'search-open';
    
    if( get_theme_mod( 'sticky_menu', 1 ) && has_nav_menu( 'secondary' ) ) {
      $classes[] = 'sticky-nav';
    }
    
    if( get_theme_mod( 'winter_mode', 0 ) ) {
      $classes[] = 'snow-on';
    }

    $blog_layout = apply_filters( 'wdb_blog_layout', get_theme_mod( 'blog_layout', 'list' ) );

    if( $blog_layout !== 'list' && ! is_singular() ) {
      $classes[] = $blog_layout . '-layout';
    }

    $sidebar_layout = apply_filters( 'wdb_sidebar_layout', get_theme_mod( 'sidear_layout' ) );

    if( $sidebar_layout ) {
      $classes[] = 'sidebar-' . $sidebar_layout;
    }

    $sidebar_position = apply_filters( 'wdb_sidebar_position', get_theme_mod( 'sidebar_position' ) );

    if( $sidebar_position == 'right' ) {
      $classes[] = 'sidebar-right';
    }

    if( get_bloginfo( 'language' ) == 'ja' ) {
      $classes[] = 'ja-language';
    }

    return $classes;
  }
  
  /**
   * This code will adda a styleselect drop down menu on the second row of the buttons.
   *
   * @since Wild Book 1.4
   *
   * @return array Editor buttons array.
   */    
  public function wpb_mce_buttons_2( $buttons ) {
    array_unshift($buttons, 'styleselect');
    return $buttons;
  }
  
  /**
   * Adding drop down menu with editor styles.
   *
   * @since Wild Book 1.4
   *
   * @return array Style formats array.
   */   
  function insert_formats_to_editor( $init_array ) {  
  // Define the style_formats array
    $style_formats = array(  
      // Each array child is a format with it's own settings
      array(  
        'title' => esc_html__( 'Uppercase', 'wildbook' ),  
        'inline' => 'span',  
        'classes' => 'uppercase-text',
        'wrapper' => true
      ),
      array(  
        'title' => esc_html__( 'Gray', 'wildbook' ),  
        'inline' => 'span',  
        'classes' => 'gray-text',
        'wrapper' => true
      ),
      array(  
        'title' => esc_html__( 'Small', 'wildbook' ),  
        'inline' => 'small',  
        'classes' => 'small-text',
        'wrapper' => true
      ),
      array(  
        'title' => esc_html__( 'Large', 'wildbook' ),  
        'inline' => 'span',  
        'classes' => 'large-text',
        'wrapper' => true
      ),
      array(  
        'title' => esc_html__( 'Large+', 'wildbook' ),  
        'inline' => 'span',  
        'classes' => 'large-p-text',
        'wrapper' => true
      ),
      array(  
        'title' => esc_html__( 'Large++', 'wildbook' ),  
        'inline' => 'span',  
        'classes' => 'large-pp-text',
        'wrapper' => true
      ),
      array(  
        'title' => esc_html__( 'Framed', 'wildbook' ),  
        'block' => 'div',  
        'classes' => 'text-box',
        'wrapper' => true
      )
    );  
    // Insert the array, JSON ENCODED, into 'style_formats'
    $init_array['style_formats'] = json_encode( $style_formats );  
    
    return $init_array;  
  } 
  
  /**
   * If a page and portfolio post type have same slug, need make new unique slug for page.
   *
   * @since Wild Book 1.4.3
   */     
  public function portfolio_is_bad_hierarchical_slug( $is_bad_hierarchical_slug, $slug, $post_type, $post_parent ) {
    if ( !$post_parent && $slug == 'portfolio' )
        return true;
    return $is_bad_hierarchical_slug;
  }
  
  /**
   * If a page and portfolio post type have same slug, need make new unique slug for page.
   *
   * @since Wild Book 1.4.3
   */       
  public function portfolio_is_bad_flat_slug( $is_bad_flat_slug, $slug, $post_type ) {
    if ( $slug == 'portfolio' )
        return true;
    return $is_bad_flat_slug;
  }
  
  /**
   * Changing default archive title for portfolio post type.
   *
   * @since Wild Book 1.4.3
   */    
  public function archive_title( $title ) {
    if( is_post_type_archive( 'portfolio' ) ) {
      return esc_html__( 'Portfolio', 'wildbook' );
    }

    if ( is_category() || is_tax( 'portfolio_category' ) ) {
      return single_cat_title( '', false );
    }

    return $title;
  }

  /**
   * Modification of the default fields of the comment form..
   *
   * @since Wild Book 1.6.4
   */  
  public function comment_form_default_fields( $fields ) {
    $commenter = wp_get_current_commenter();

    $req      = get_option( 'require_name_email' );
    $aria_req = ( $req ? " aria-required='true'" : '' );
    $html_req = ( $req ? " required='required'" : '' );
    $html5    = 'html5';

    $fields['author'] = '
      <p class="comment-form-author">
        <input id="author" name="author" type="text" placeholder="' . esc_attr__( 'Name', 'wildbook' ) . ( $req ? ' *' : '' ) . '" value="' . esc_attr( $commenter['comment_author'] ) . '" size="30"' . $aria_req . $html_req . ' />
      </p>';

    $fields['email'] = '
      <p class="comment-form-email">
        <input id="email" name="email" ' . ( $html5 ? 'type="email"' : 'type="text"' ) . ' placeholder="' . esc_attr__( 'Email', 'wildbook' ) . ( $req ? ' *' : '' ) . '" value="' . esc_attr(  $commenter['comment_author_email'] ) . '" size="30" aria-describedby="email-notes"' . $aria_req . $html_req  . ' />
      </p>';

    $fields['url'] = '
      <p class="comment-form-url">
        <input id="url" name="url" ' . ( $html5 ? 'type="url"' : 'type="text"' ) . ' placeholder="' . esc_attr__( 'Website', 'wildbook' ) . '" value="' . esc_attr( $commenter['comment_author_url'] ) . '" size="30" />
      </p>';

    return $fields;
  }
}

$wild_book_setup = new Wild_Book_Setup();

add_action( 'after_setup_theme', array( $wild_book_setup, 'setup' ), 16 );

/**
 * Customizer additions.
 *
 * @since Wild Book 1.0
 */
require get_template_directory() . '/inc/customizer.php';

/**
 * Custom template tags for this theme.
 *
 * @since Wild Book 1.0
 */
require get_template_directory() . '/inc/template-tags.php';

/**
 * Post metaboxes.
 *
 * @since Wild Book 1.0
 */
require get_template_directory() . '/inc/metaboxes.php';

/**
 * Custom comments walker.
 *
 * @since Wild Book 1.0
 */
require get_template_directory() . '/inc/comments-walker.php';

/**
 * Add fields to category form.
 *
 * @since Wild Book 1.4
 */
require get_template_directory() . '/inc/category-form-fields.php';

/**
 * Adding additional types of galleries.
 *
 * @since Wild Book 1.6
 */
require get_template_directory() . '/inc/class-wild-book-custom-galleries.php';

/**
 * TGM Plugin Activation
 *
 * @since Wild Book 1.6
 */
require get_template_directory() . '/inc/tgmpa/tgmpa.php';